#!/usr/bin/perl

use strict;
use warnings;
use utf8;
use DBI;   
use HTML::Tree;
use Config::IniFiles;
use Getopt::Long;    
use Sys::Hostname;       
use Text::Iconv;          
use Digest::MD5;
use Time::Local;
use vars qw($configfile $quiet $help $version $verbose $clear);




#####zmiany tutaj######

#### adres url do strony banku
my $url = "";
### login do banku
my $user="";
my $pass="";
### automatyczne potwierdzanie wplat zidentyfikowanych poprzez subkonto ####
my $autoconfirm = "0";
####USTAWIENIE DAT DO TESTOW, POZOSTAW PUSTE DO NORMALNEGO DZIALANIA #####
my $date_from="";
my $date_to="";



####DALEJ NIE RUSZAC#####

$ENV{'PATH'}='/sbin:/usr/sbin:/usr/local/sbin:/bin:/usr/bin:/usr/local/bin';

my $_version = '1.10.7 Sina';

my %options = (
        "--config-file|C=s"     =>      \$configfile,
        "--quiet|q"             =>      \$quiet,     
        "--help|h"              =>      \$help,      
        "--version|v"           =>      \$version,   
);                                                   

Getopt::Long::config("no_ignore_case");
GetOptions(%options);                  


if($help)
{        
        print STDERR <<EOF;
lms-bsebanknet-cashimport, version $_version
(C) 2010 Flynet, Michał Gacek & LMS Developers

-C, --config-file=/etc/lms/lms.ini      alternate config file (default: /etc/lms/lms.ini);
-h, --help                      print this help and exit;                                 
-v, --version                   print version info and exit;                              
--verbose                       run in verbose mode to troubleshoot errors;               
-q, --quiet                     suppress any output, except errors;                       

EOF
        exit 0;
}              

if($version)
{           
        print STDERR <<EOF;
lms-bsebanknet-cashimport, version $_version
(C) 2010 Flynet, Michał Gacek & LMS Developers

EOF
        exit 0;
}              

if(!$configfile)
{               
        $configfile = "/etc/lms/lms.ini";
}                                        



if(! -r $configfile)
{
        print STDERR "Fatal error: Unable to read configuration file $configfile, exiting.\n";
        exit 1;                                                                               
}

my $ini = new Config::IniFiles -file => $configfile;
print @Config::IniFiles::errors;                    

my $dbtype = $ini->val('database', 'type') || 'mysql';
my $dbhost = $ini->val('database', 'host') || 'localhost';
my $dbuser = $ini->val('database', 'user') || 'root';     
my $dbpasswd = $ini->val('database', 'password') || '';   
my $dbname = $ini->val('database', 'database') || 'lms';  
my $dbase;
my $utsfmt;
my $converter = Text::Iconv->new("ISO-8859-2", "UTF8");

if($dbtype =~ /mysql/)
{
        $dbase = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
        $utsfmt = "UNIX_TIMESTAMP()";
	$dbase->do("SET NAMES utf8");
}
elsif($dbtype eq "postgres")
{
        $dbase = DBI->connect("DBI:Pg:dbname=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
        $utsfmt = "EXTRACT(EPOCH FROM CURRENT_TIMESTAMP(0))";
}
else
{
        print STDERR "Fatal error: unsupported database type: $dbtype, exiting.\n";
        exit 1;
}

(my $ss,my $mm,my $hh,my $day,my $month,my $year) = (localtime(time - 24 * 60 * 60))[0,1,2,3,4,5,6];
$year += 1900;
$month=sprintf("%02d", $month+1);
$day=sprintf("%02d", $day);
if ($date_to eq '' && $date_from eq '') {
	$date_to="$year-$month-$day";
	$date_from="$date_to";
}


######SUBS############

sub parsetransactions($) {
	my ($row) = @_;
	my @cols = $row->find('td');
	if(defined($cols[0])) {
		if ( $cols[2]->as_text() =~ m/^ (\d+,\d+)/  ) {
			
			###OPERACJA UZNANIA !! DZIALAMY
			print " \n NOWEEEEEEEE UZNANIEEEEEEEEEEEEE \n\n";	    
			my $value = $1;
    			my $account = '';
			my $accountowner = '';
			my $optitle = '';
			my $customerid ='';
			
			###WPLATA W ODDZIALE BANKU		
			if($cols[1]->as_XML =~ m/^<.*?><.*?>WP.*?<span.*?>(.*?)<\/span><br \/>(.*?)(za:\d{2} \d{8} \d{4} \d{4} \d{4} (\d{4})|<\/td>)/) {
				$account = 'WPLATA W BANKU';
				$accountowner = $1;
				$optitle = $2;
				if(defined($4)) {
					$customerid = $4;
				}
			}
			
			###TYPOWY PRZELEW
			else {
				$cols[1]->as_XML =~ m/(.*?)<.*\/>(.*?)<.*?\/><.*?>(.*?)<.*?><.*?\/>(.*?)(za:\d{2} \d{8} \d{4} \d{4} \d{4} (\d{4})|<.*>)/;
				$account = $2;
				$accountowner = $3;
				$optitle = $4;
				$customerid ='';
	
				if(defined($6)) {
					$customerid = $6;
				}
			}
			
			###JEZELI TYTUL BYL ZBYT DLUGI OTWIERAMY SZCZEGOLY TRANZAKCJI
			if ( $cols[1]->as_XML() =~ m/ \.\.\. <.*?>/  ) {
				$cols[4]->as_XML() =~ m/href=\"(.*?)\"/;
				my $details = `curl -L "${url}$1" 2>/dev/null `;
				my $dettree = HTML::TreeBuilder->new;
				$dettree->parse($details);
				$dettree->as_XML() =~ m/ytu. operacji:<.*?><.*?>(.*?)<\/td>/;
				$optitle = $converter->convert("$1");		
				$optitle =~ m/(.*?)(za:\d{2} \d{8} \d{4} \d{4} \d{4} (\d{4}))?<br \/> $/;
				if(defined($3)) {
					$customerid = $3;
				}
				$optitle = $1;
			}
			
			###CZYSCIMY TYTUL Z NEWLINEOW
			$optitle =~ s/\<br \/\>//g;
			
			###wyciagamy date ksiegowania i konwertujemy do timestamp
			$cols[0]->as_text() =~ m/(\d{4}-\d{2}-\d{2}) ((\d{4})-(\d{2})-(\d{2}))/;
			my $date = timelocal(0,0,0,$5,$4-1,$3);
			
			###USTALAMY FORMAT OPISU OPERACJI
			my $description = "$account<br><b>$accountowner</b><br><i>$optitle</i>\n";
			print "data operacji: $1\ndata ksiegowania: $2\nidklienta: $customerid\n$account\n$accountowner\n$optitle\n\n";
			
			###GENERUJEMY HASH TRANZAKCJI
			my $dx = Digest::MD5->new();
			$dx->add($date);
			$dx->add($value);
			$dx->add($description);
			$dx->add($customerid);
			my $hash = $dx->hexdigest();
	    
			###SPRAWDZAMY CZY TRANZAKCJA JEST NOWA
			my $dbq = $dbase->prepare("SELECT id FROM cashimport WHERE hash = '$hash'");
			$dbq->execute();
			if (!$dbq->fetchrow()) {
				my $customer = '';
				my $confirm = 0;
				
				###JEZELI WYKRYTO ID KLIENTA USTAWIAMY POLE CUSTOMER WG DANYCH Z BAZY
				if (!$customerid eq '') {
					$dbq = $dbase->prepare("SELECT CONCAT(UPPER(lastname),' ',name) FROM customers where id='$customerid'");
					$dbq->execute();
					$customer = $dbq->fetchrow();		
					
					### JEZELI WLACZONE AUTOPOTWIERDZANIE TWORZYMY OPERACJE TABELI CASH
					if($autoconfirm) {
						$dbq = $dbase->prepare("INSERT INTO cash VALUES ('', '$date', '1', '0', '$value', '0', '$customerid', '$description', '0', '0');");
						$dbq->execute();
						$confirm = 1;
					}
				}
				
				###ZAPISUJEMY INFORMACJE DO TABELI CASHIMPORT
				$dbq = $dbase->prepare("INSERT INTO cashimport VALUES ('', '$date', '$value', '$customer', '$description', '$customerid', '$hash', '$confirm');");
				$dbq->execute();
				
				
			}
			else {
				print STDERR "ERROR: MAM JUZ  TAKA WPLATE HASH: $hash\n\n";
			}
	    
	    
		}
		else {
			print "\n NOWYYYYYYYYY        ROZCHODDDDDDD\n\n";
		}
	}    
}









###NAGLOWEK
print "\n\n\n############# INSTNACJA URUCHOMIONA: $year/$month/$day " . sprintf("%02d",$hh). ":" . sprintf("%02d",$mm) .  " ###################### \n\n\n";
print "IMPORT TRANZAKCJI Z OKRESU: $date_from - $date_to !!!!!!!!!!!\n\n\n";

#zalogowanie do systemu
my $curlpage = $url . '/Auth/Login.do';
my $response = `curl -d "username=${user}&password=${pass}" -L "$curlpage" 2>/dev/null `;
my $tree = HTML::TreeBuilder->new;
$tree->parse($response);

#zdobycie numerku sesji
my ($tag) = $tree->look_down('name', 'MainFrame');
$curlpage = $url .  $tag->attr('src'); 
my $jsid = $tag->attr('src');
$jsid =~ s/\/Operation\/CheckNW.do//;

#potwierdzenie logowania i wybranie rachunku
$response = `curl -d "username=${user}&password=${pass}" -L "$curlpage" 2>/dev/null `;
$tree->parse($response);
($tag) = $tree->look_down('title', 'Wybierz rachunek');
$curlpage = $url .  $tag->attr('href'); 
$response = `curl -L "$curlpage" 2>/dev/null `;

#przejscie do filtra historii
$tree->parse($response);
$curlpage = $url . '/Operation/RangePrepare.do' . $jsid;
$response = `curl -L "$curlpage" 2>/dev/null `;

#wybranie filtra historii
$tree->parse($response);
$curlpage = $url . '/Operation/RangeSelect.do' . $jsid;
$response = `curl -d "MODE=2&DATE_FROM=${date_from}&DATE_TO=${date_to}" -L "$curlpage" 2>/dev/null `;

#konwersja do utf8
$converter = Text::Iconv->new("ISO-8859-2", "UTF8");
$response = $converter->convert("$response");

#parsowanie odpowiedzi
$tree = HTML::TreeBuilder->new;
$tree->parse($response);

my @tables = $tree->look_down('class', 'main_tab');

###### czy byly jakies tranzakcje #######
if (defined($tables[1])) {

	my @rows = $tables[1]->find('tr');
    
	###### parsujemy tranakcje
	foreach (@rows) {
		&parsetransactions($_);
	}

	##### sprawdzamy czy jest kolejna strona ####
	($tag) = $tree->look_down('value', 'next');
	while (defined $tag) { 
    	        print "###PARSUJE KOLEJNA STRONE### \n"; 
        
		### szukamy informacji do prejscia do kolejnej strony ####
    		$tree->parse($response);
		my $form = $tree->look_down('name', 'onf');    
    		my @inputs = $form->descendants();
		my $vars ='';
		foreach (@inputs) {
			$vars = $vars . '&'. $_->{'name'} . '=' . $_->{'value'};
		}
	
		### przechoddzimy do kolejnej strony
		$curlpage = $url . '/Operation/List.do' . $jsid;
    		$response = `curl -d "MODE=2&DATE_FROM=${date_from}&DATE_TO=${date_to}${vars}next" -L "$curlpage" 2>/dev/null `;

		#konwersja do utf8
		$response = $converter->convert("$response");

		#parsowanie odpowiedzi
		$tree = HTML::TreeBuilder->new;
		$tree->parse($response);
		@tables = $tree->look_down('class', 'main_tab');
		@rows = $tables[1]->find('tr');
	
		#### parsujemy tranzakcje
    		foreach (@rows) {
    			&parsetransactions($_);
		}

		#### sprawdzamy czy jest kolejna strona
		($tag) = $tree->look_down('value', 'next');
	}
}
else {
	print "Nie bylo tranzakcji w tym dniu";
}
print "\n############## KONIEC CZYTANIA!!!!!! ############### \n\n\n\n";

